
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% filter_vmr_in_matlab.m
% Script to filter a BrainVoyager VMR project in Matlab
% 
% Works with BrainVoyager QX 1.9 and VMR versions higher than 1.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[FileName,PathName] = uigetfile('*.vmr','Please select a VMR file');
vmrfilename = [PathName FileName];
fid = fopen(vmrfilename, 'rb');
version = fread(fid, [1 1], 'uint16'); 
dimx = fread(fid, [1 1], 'uint16'); 
dimy = fread(fid, [1 1], 'uint16'); 
dimz = fread(fid, [1 1], 'uint16');
data = fread(fid, [1 dimx*dimy*dimz], 'uchar');
rest = fread(fid);
fclose(fid);
vmrdata = reshape(data, dimx, dimy, dimz);

%  H = FSPECIAL('log',HSIZE,SIGMA) returns a rotationally symmetric
%     Laplacian of Gaussian filter of size HSIZE with standard deviation
%     SIGMA (positive). HSIZE can be a vector specifying the number of rows
%     and columns in H or a scalar, in which case H is a square matrix.
%     The default HSIZE is [5 5], the default SIGMA is 0.5.

% show filters
figure;
img = vmrdata(:,:,floor(dimx/2));
img = rot90(img, 3);
subplot(2,2,1);imagesc(img);title('Original Image'); 
H = fspecial('log',[5 5],.5);
Laplacian = imfilter(img,H);
subplot(2,2,2);imagesc(Laplacian);title('Laplacian of Gaussian filter');
cannyimg = edge(img,'canny',.15);
subplot(2,2,3);imagesc(cannyimg);title('Canny filter (0.15)');
H = fspecial('sobel');
sobelimg = imfilter(img,H);
subplot(2,2,4);imagesc(sobelimg);title('Sobel filter');
colormap gray;

filterlist = {'Laplacian', 'Canny', 'Sobel'};
[filter,ok] = listdlg('PromptString','Select a filter:',...
    'SelectionMode','single',...
    'Name', 'Filter VMR',...
    'ListString',filterlist);

H_log = fspecial('log',[5 5],.5);
H_sobel = fspecial('sobel');            

if filter == 2
    prompt = {'Enter canny filter value:'};
    dlg_title = 'Filter VMR';
    num_lines= 1;
    def     = {'0.15'};
    answer  = inputdlg(prompt,dlg_title,num_lines,def);
    filterval = answer{1};
end

% apply filter and write image
[pathstr,name,ext,versn] = fileparts(vmrfilename);
filteredimgname = fullfile(pathstr, [name '_filtered' ext]);
if ok == 1
    % write VMR header
    fid = fopen(filteredimgname,'w');
    fwrite(fid,version,'ushort');     
    fwrite(fid,dimx,'ushort'); 
    fwrite(fid,dimy,'ushort'); 
    fwrite(fid,dimz,'ushort'); 
    % write VMR image volume
    for slicenr=1:len(3)
        img = vmrdata(:,:,slicenr);
        if filter==1
            filteredimg = imfilter(img,H_log);
        elseif filter ==3
            filteredimg = imfilter(img, H_sobel);
        elseif filter == 2
            filteredimg = edge(img,'canny',filterval);       
        else 
            break;
        end
        fwrite(fid,filteredimg,'uchar'); 
    end
    % write rest of VMR header
    fwrite(fid,rest); 
    fclose(fid);
end

% show in BrainVoyager QX
bvqx = actxserver('BrainVoyagerQX.BrainVoyagerQXInterface');
filteredvmr = bvqx.OpenDocument(filteredimgname);
bvqx.ShowLogTab;
bvqx.PrintToLog(['Filter: ' filterlist{filter}]);