
function writeTrf(trftype, matrix, coordsys, sourcefile, targetfile, varargin)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% writeTrf.m
% Inputparameters:
% 1.  	trftype: 1= rigidbody; 2=affine (f.e. tosag and isovoxel); 4=tal; 5=untal
% 2.  	4 x 4 matrix (Matlab format: column oriented), 1 x 16 vector [col1 col2 col3 col4] or mat filename (SPM)
% 3.  	coordsys: 0 is BV internal; 1 is BV sys; 2 is Talairach.     
% 4.  	sourcefile; in case of transformation type 2, optional. 
% 5.  	targetfile (trftype=1)
% 6. 	slices; number of slices of functional data(trftype=1)
% 7. 	functhick: slice thickness of functional images (trftype=1)
% 8. 	gap; slice gap of functional data(trftype=1)
% 9. 	fmr3d; 2 or 3 (trftype=1)
% 10. 	alignstep; 1: initial alignment; 2: fine alignment (trftype=1)
% 11. 	matxtra: 4 x 4 matrix specifying a transformation of the anatomical data (trftype=1)
% 
% H. Breman, Brain Innovation, 2005
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

if trftype==1
    if nargin < 10, disp('Need at least parameters for rigid body transformation file.'); return; end
    if nargin < 11 
        xtratrf = 0; 
    else 
        matxtra = varargin(5)
        xtratrf = 1;
    end
        slices = varargin(1)
        gap = varargin(2)
        fmr3d = varargin(3)
        alignstep = varargin(4)
elseif trftype==2 
    if nargin < 2, disp('Need at least 2 parameters for affine transformation file.'); return; end
    if nargin < 5, targetfile = fullfile(pwd,'somefile_TRF.vmr'); end
    if nargin < 4, sourcefile = fullfile(pwd,'somefile.vmr'); end
    if nargin < 3, coordsys = 2; end % assume TAL or MNI space 
    xtratrf = 0;
end
fileversion = 5;
[srcPath,sourcePrt,srcExt,srcVersn] = fileparts(sourcefile); 

if isnumeric(matrix)
    [rows, cols] = size(matrix);
    if rows==4 && cols==4
        mat1 = reshape(matrix, 1,[]);
    elseif rows*cols~=16
        disp('Wrong number of parameters in matrix');
        return;
    else
        mat1 = matrix;
    end
elseif exist(matrix)==2
    s = load(matrix,'M');
    mat1 = reshape(s.M, 1,[]); %s.M;
else %if exist(matfile)~=2
    disp('Unrecognized input for matrix parameter');
    return;
end  

if trftype ==1
    [trgPath,targetPrt,trgExt,trgVersn] = fileparts(targetfile); 
    trfName = fullfile(srcPath, [targetPrt 'TO' sourcePrt '.trf']);        
else %if trftype ==2
    trfName = fullfile(srcPath, [sourcePrt '.trf']);
end

trfFid = fopen(trfName,'wt'); 
fprintf(trfFid, '\n%s\t%s\n', 'FileVersion:', num2str(5)); 
fprintf(trfFid, '\n%s\t%s\n\n', 'DataFormat:', 'Matrix');
fprintf(trfFid, '%3.16f\t%3.16f\t%3.16f\t%3.16f\n', mat1(1,1), mat1(1,5), mat1(1,9), mat1(1,13)); 
fprintf(trfFid, '%3.16f\t%3.16f\t%3.16f\t%3.16f\n', mat1(1,2), mat1(1,6), mat1(1,10), mat1(1,14)); 
fprintf(trfFid, '%3.16f\t%3.16f\t%3.16f\t%3.16f\n', mat1(1,3), mat1(1,7), mat1(1,11), mat1(1,15)); 
fprintf(trfFid, '%3.16f\t%3.16f\t%3.16f\t%3.16f\n\n', mat1(1,4), mat1(1,8), mat1(1,12), mat1(1,16));
fprintf(trfFid, '%s\t%s\n', 'TransformationType:', num2str(trftype)); 
fprintf(trfFid, '%s\t%s\n\n', 'CoordinateSystem:', num2str(coordsys)); 
if trftype==1
    fprintf(trfFid, '%s\t%s\n', 'NSlicesFMRVMR:', num2str(slices)); 
    fprintf(trfFid, '%s\t%s\n', 'SlThickFMRVMR:', num2str(functhick)); 
    fprintf(trfFid, '%s\t%s\n', 'SlGapFMRVMR:', num2str(gap));
    fprintf(trfFid, '%s\t%s\n', 'CreateFMR3DMethod:', num2str(fmr3d));
    fprintf(trfFid, '%s\t%s\n\n', 'AlignmentStep:', num2str(alignstep));
    fprintf(trfFid, '%s\t%s\n\n', 'ExtraVMRTransf:', num2str(xtratrf));
    if xtratrf 
        fprintf(trfFid, '%3.16f\t%3.16f\t%3.16f\t%3.16f\n', matxtra(1,1), matxtra(1,5), matxtra(1,9), matxtra(1,13)); 
        fprintf(trfFid, '%3.16f\t%3.16f\t%3.16f\t%3.16f\n', matxtra(1,2), matxtra(1,6), matxtra(1,10), matxtra(1,14)); 
        fprintf(trfFid, '%3.16f\t%3.16f\t%3.16f\t%3.16f\n', matxtra(1,3), matxtra(1,7), matxtra(1,11), matxtra(1,15)); 
        fprintf(trfFid, '%3.16f\t%3.16f\t%3.16f\t%3.16f\n\n', matxtra(1,4), matxtra(1,8), matxtra(1,12), matxtra(1,16));
    end
end
fprintf(trfFid, '%s\t%s%s%s\n', 'SourceFile:', '"', sourcefile, '"'); 
fprintf(trfFid, '%s\t%s%s%s\n', 'TargetFile:', '"', targetfile, '"');
fclose(trfFid);

return;
